from math import ceil
import pandas as pd
import os
import json
from datetime import datetime

from tables import HDF5ExtError

from .configUtil import _addVideoRecord


def scanH5files(h5files):
    '''
    For each videoID:
    returns a tuple: 
        Critical frame: the frame where two mice are present(with no NAN)
        Number of frames: the number of frames in the original video
    Input:
        h5files: a list of paths or the directory
    '''
    res = {}
    basepath = ""
    if type(h5files) is str and os.path.isdir(h5files):
        basepath = h5files
        h5files = os.listdir(h5files)
    else:
        if not type(h5files) is list:
            print("Input h5files is a list!")
            return res

    for h5file in h5files:
        try:
            data = pd.read_hdf(os.path.join(basepath,h5file))
        except FileNotFoundError:
            print("File {} does not exists!".format(h5file))
            continue
        except HDF5ExtError:
            continue

        # check data, get the critical frame
        crtFrame = 0
        
        numFrames = len(data.index)
        for i in range(len(data.index)):
            if not data.iloc[i].isnull().any():
                crtFrame = i
                break
        videoID = h5file[:os.path.basename(h5file).find("DLC")]
        res[videoID] = (crtFrame, numFrames)

    return res

def getFrameCount(h5file):
    '''
    returns the number of frames based on the DLC result h5file
    returns -1 if error
    '''
    try:
        data = pd.read_hdf(h5file)
        return len(data.index)

    except FileNotFoundError:
        print("File {} does not exist".format(h5file))
        return -1
    except HDF5ExtError:
        print("HDF5ExtError")
        return -1

def batch_dynamic_cropVideo(config,videos,h5files,fps,videoType="avi",destDir=None,dontCut=False):
    '''
    Main function for dynamic cutting videos: 
    1. extract the portion in the video where two mice are present based on DLC h5file
    2. Record video information in the configuration file
    '''
    if not dontCut and os.system("ffmpeg -h") != 0:
        print("Please install ffmpeg!")
        exit()
    
    if not destDir:
        destDir = os.path.join(os.getcwd(),"preprocessed")
    if not os.path.isdir(destDir):
        if not os.path.exists(destDir):
            os.mkdir(destDir)
        else:
            print("Error: {} is a file.".format(destDir))
            exit()
        
    scanned_dict = scanH5files(h5files)
    videopath = os.path.join(videos)
    videos = os.listdir(videos)
    
    for video in videos:
        videoID = video.split(".")[0]
        if videoID in scanned_dict:
            crtFrame, numFrame = scanned_dict[videoID]
            start_crop_time = ceil(crtFrame / fps)
            
            destpath = os.path.join(destDir,"cut_"+video)
            if not dontCut:
                ret = os.system("ffmpeg -i {} -ss {} -c copy {}".format(os.path.join(videopath,video), start_crop_time,destpath))

            if dontCut or ret == 0:
                _addVideoRecord(config,videoID,start_crop_time,numFrame)

    print("Done.")



"""
Utilities for manipulating the configuration file
"""

import json
import datetime
import os

'''
Configuration File:
{   
    "Project_Name": 
    "Experimenter":
    "Date": 
    "Videos":
    [   
        {
        "ID": 399
        "start_time": 3
        "FrameCount": 12382
        }
    ]
}
'''

def initConfig(Dirpath,projectName=None,experimenter=None):
    data = {}
    data["Project_Name"] = projectName if projectName else "newProject"
    data["Experimenter"]= experimenter if experimenter else "experimenter1"
    data["Date"] = datetime.datetime.now().strftime("%b/%d/%Y")
    data["Videos"]=[]

    jsonPath = os.path.join(Dirpath,"config.json")
    print(jsonPath)
    with open(jsonPath,"w+") as jsonfile:
        json.dump(data,jsonfile)

# add video record to config
def _addVideoRecord(jsonPath,videoID,start_crop_time,originalFrameCount):
    with open(jsonPath,"r+") as jsonfile:
        data = json.load(jsonfile)
        data["Videos"].append({"ID":videoID,"start_time":start_crop_time,"FrameCount":originalFrameCount})            
        jsonfile.truncate(0)
        jsonfile.seek(0)
        json.dump(data,jsonfile)


def readConfig(config):
    data = None
    try:
        with open(config,"r") as jsonfile:
            data = json.load(jsonfile)
    except:
        print("Error in reading config file.")
        exit(1)
    return data

def readVideoData(config):
    """
    Read Config, generate a dictionary containing video Info: 
        {VideoID: {"ID": .., "start_time": .., "FrameCount": ..}}
    """

    videoData = readConfig(config)["Videos"]
    res = {}
    for item in videoData:
        res[item["ID"]] = item

    return res



"""
Utilities for processing simba output to generate the desired Interaction vector
"""

from .configUtil import readVideoData
import os
import csv


def finalizeSimBaOutput(config, simbaFiles, classifier ,destDir = None):
    """
    Generate the final interaction vector
    Input:
        simbaFiles: a list or a directory of path to simba csv output
        classifier: a list of classifer names (e.g. isInteraction)
    """
    videoData = readVideoData(config)

    if type(simbaFiles) is str and os.path.isdir(simbaFiles):
        basepath = simbaFiles
        simbaFiles = os.listdir(simbaFiles)
    else:
        if not type(simbaFiles) is list:
            print("Input simbaFile should be either a list or a directory!")
            return None

    if not destDir:
        destDir = os.path.join(os.getcwd(),"final_result")
    if not os.path.isdir(destDir):
        if not os.path.exists(destDir):
            os.mkdir(destDir)
        else:
            print("Error: {} is a file.".format(destDir))
            exit()

    # get indexes of wanted columns
    wantedColumnNames = set()
    for cl in classifier:
        wantedColumnNames.add("Probability_" + cl)
        wantedColumnNames.add(cl)
    
    wantedColumns = []

    for simbaCSV in simbaFiles:
        if not simbaCSV.split('.')[-1] in {'csv','CSV'}:
            continue
        
        videoID = simbaCSV[4:-4]

        try:
            with open(os.path.join(basepath, simbaCSV)) as csvFile:
                data = list(csv.reader(csvFile))
                index = data[0]
                data = data[1:]
        except FileNotFoundError:
            print("File {} does not exists!".format(simbaCSV))
            continue

        if not wantedColumns:
            wantedColumns = [0]
            for i,name in enumerate(index):
                if name in wantedColumnNames:
                    wantedColumns.append(i)

        # reconstruct a new csv file
        originalVideoFrameCount = int(videoData[videoID]['FrameCount'])
        offset = originalVideoFrameCount - len(data)

        for i in range(len(data)):
            data[i] = [data[i][x] for x in wantedColumns]
            data[i][0] = int(data[i][0]) + offset
        
        index = [index[x] for x in wantedColumns]
        index[0] = "FrameNumber"
        
        csvPath = os.path.join(destDir,"final_"+videoID+".csv") 
        with open(csvPath,"w") as f:
            wr = csv.writer(f)
            wr.writerow(index)
            wr.writerows(data)
        print("Wrote {}.".format(csvPath))
    
    print("Done.")
